<?php

/*
 * the home page
 */
require_once("include.php");

InputPage::$method = "post";

class ButtonCreateReport extends InputPageButtonOnlyAction {

    public function ButtonCreateReport($name, $label) {
        parent::__construct($name, $label);
        $this->anyway = false;
    }
    
    static function repOrderCols(&$q, &$oq, &$reportColumns, $groupBy, $sortBy, $sortOrder, $displayedDuration) {
        $cols = ReportingConfigConsts::getColNames(true);
        $cols[ReportingConfigConsts::colCLowerCn] = "";
        $cols[ReportingConfigConsts::colCdCallbackH323] = "";
        $cols[ReportingConfigConsts::colCdCallbackE164] = "";
        $cols[ReportingConfigConsts::colCdCallbackDn] = "";
        $cols[ReportingConfigConsts::colCdCallbackTime] = "";
        $cols[ReportingConfigConsts::colCdCallbackTimeUnix] = "";
        $cols[ReportingConfigConsts::colUnixTime] = "";
        foreach($cols as $col => $name) {
            if ($col == ReportingConfigConsts::colLocalTime || $col == ReportingConfigConsts::colCdCallbackTime) {
                $q .= self::getDateCol($col).", ";
            }
            else {
                $q .= "$col, ";
            }
            $reportColumns[] = $col;
        }
        $q = rtrim($q, ", ");
        if ($groupBy == ReportingConfigConsts::colLocalTime) {
            $q .= ", to_char(local_stamp, '" . self::getDateFormat() . "') as datum, to_char(local_stamp, '" . self::getTimeFormat() . "') as time";
            $q .= ", date_part('epoch', date_trunc('day', local_stamp)) AS ".ReportingConfigConsts::colDateSort;
            $reportColumns[] = "datum";
            $reportColumns[] = "time";
            $reportColumns[] = ReportingConfigConsts::colDateSort;
        }
        $q .= " FROM cdrs cdr JOIN cdr_properties cdrp ON cdr.id = cdrp.cdr_id ";
        
        $sortByTmp = ReportingConfigConsts::colUnixTimeSort;
        $sortByTmp2 = ReportingConfigConsts::colUnixTimeSort;
        switch($sortBy) {
            case ReportingConfigConsts::colCCnUser:
                $sortByTmp = ReportingConfigConsts::colCLowerCn;
                break;
            case ReportingConfigConsts::colLocalTime:
                $sortByTmp2 = ReportingConfigConsts::colCLowerCn;
                break;
            case ReportingConfigConsts::colAlertDuration:
                $sortByTmp = $sortBy;
                break;
            case ReportingConfigConsts::colBillingDuration:
            case ReportingConfigConsts::colConnDuration:
            case ReportingConfigConsts::colCallDuration:
                $sortByTmp = $displayedDuration;
                break;
        }
        switch($groupBy) {
            case ReportingConfigConsts::colCCnUser:
                $oq = ReportingConfigConsts::colCLowerCn . " ";
                if($sortByTmp == ReportingConfigConsts::colCLowerCn) {
                    $oq .= " $sortOrder, ";
                }
                else {
                    $oq .= " ASC, ";
                }
                break;
            case ReportingConfigConsts::colLocalTime:
                $oq = ReportingConfigConsts::colDateSort;
                if($sortByTmp == ReportingConfigConsts::colUnixTimeSort) {
                    $oq .= " $sortOrder, ";
                }
                else {
                    $oq .= " ASC, ";
                }
                break;
            default:
                $oq = "";
                break;
        }
        $oq .= "$sortByTmp $sortOrder, $sortByTmp2";
    }
    
    static function getDateCol($colName) {
        return "to_char($colName, '" . self::getDateFormat() . " " . self::getTimeFormat() . "') AS $colName";
    }

    static function getTimeFormat() {
        return "HH24:MI:SS";
    }

    static function getDateFormat() {
        return tl::tlx(/* date format, args are 1: year, 2: month, 3: day */ '{$arg[2]}/{$arg[3]}/{$arg[1]}', "YYYY", "MM", "DD");
    }

    /* There must be at least one condition */

    static function repOrderCondFilter($filt, &$qc, &$isEvent = true) {
        $index_conds = 0;
        $conds = $filt->conds;
        $nl = count($filt->conds);
        if ($nl > 0)
            $qc .= "((";
        foreach ($conds as $cond) {
            if ($cond->name == ReportingConfigConsts::colCdE164 OR $cond->name == ReportingConfigConsts::colCdDn) {
                $isEvent = true;
                $qc .= "(" . $cond->name . " ILIKE '" . $cond->value . "' AND (ev.order_index=0 OR (ev.order_index=1 AND ev.msg NOT IN ('cf-to', 'cf-from', 'pickup-from', 'scf', 'sct', 'transfer-from', 'transfer-to', 'transfer-remote')))) ";
            } elseif ($cond->name == ReportingConfigConsts::colGroups) {
                $qc .= "cdrp_cdr_id_groups ILIKE '%{" . inno_escape($cond->value) . "}%' ";   // use {} around value to allow matches like search for tapi which musn't match tapi2
            } else {
                $qc .= $cond->name . " ILIKE '" . $cond->value . "' ";
            }

            if ($index_conds + 1 == $nl) {
                $qc .= ")) ";
                break;
            }

            if ($conds[$index_conds]->type == $conds[$index_conds + 1]->type)
                $qc .= "OR ";
            else
                $qc .= ") AND (";

            $index_conds++;
        }
        if ($filt->callDirection != ReportingConfigConsts::callDirectionAll) {
            $filtDirConds = explode("|", $filt->callDirection);
            if (count($filtDirConds)) {
                if ($qc != "") {
                    $qc .= "AND ";
                }
                $qc .= "(";
                foreach ($filtDirConds as $filtDirCond) {
                    switch ($filtDirCond) {
                        case ReportingConfigConsts::callDirectionCf:
                        case ReportingConfigConsts::callDirectionCt:
                            $qc .= "cdrp.cdrp_call_flow ILIKE '%{s$filtDirCond,%' OR ";
                            break;
                        case ReportingConfigConsts::callDirectionIn:
                        case ReportingConfigConsts::callDirectionOut:
                            $qc .= "cdr_dir ILIKE '$filtDirCond' OR ";
                            break;
                    }
                }
                $qc = rtrim($qc, " OR ");
                $qc .= ") ";
            }
            unset($filtDirConds);
        }

        //if($filt->callType != ReportingConfigConsts::callingStateAll) $qc .= "AND (lower(cdrp.cdrp_type) LIKE lower('".$filt->callType."')) ";
        $callLength = (int) $filt->callingDuration;
        $alertDuration = (int) $filt->alertDuration;
        if ($filt->callingState != ReportingConfigConsts::callTypeAll) {
            $filtStateConds = explode("|", $filt->callingState);
            if ($qc != "")
                $qc .= "AND ";
            $qc .= "(";
            $first = true;
            foreach($filtStateConds as $state) {
                if (!$first)
                    $qc .= " OR ";
                else 
                    $first = false;
                switch($state) {
                    case ReportingConfigConsts::callingStateBusy:
                        $qc .= "(cause_code = " . ReportingConfigConsts::codeUserBusy . ")";
                        break;
                    case ReportingConfigConsts::callingStateNoChannel:
                        $qc .= "(cause_code = " . ReportingConfigConsts::codeNoCircChAv . " OR cause_code = " . ReportingConfigConsts::codeReqNoCircChAv . ")";
                        break;
                    case ReportingConfigConsts::callingStateConn:
                        $qc .= "(cdrp.conn_duration > 0)";
                        break;
                    case ReportingConfigConsts::callingStateAlert:
                        $qc .= "(cdrp.alert_duration > 0 AND cdrp.call_duration = 0)";
                        break;
                }
            }
            $qc .= ") ";
            unset($filtStateConds);
        } 
        if($callLength != 0) {
            if ($qc != "") {
                $qc .= "AND ";
            }
            $qc .= "(cdrp.conn_duration " . $filt->callingDurationCompare . " " . $callLength . ") ";
        }
        if($alertDuration != 0) {
            if ($qc != "") {
                $qc .= "AND ";
            }
            $qc .= "(cdrp.alert_duration " . $filt->alertDurationCompare . " " . $alertDuration . ") ";
        }
    }

    static function parseFilters($tmpFilters, &$baseFilters, $groupBy) {
        global $reportingSession;
        $filters = array();
        foreach ($tmpFilters as $filterId => $filterName) {
            $filter = ReportingMethods::getFilter($filterId, "", $error);
            if($filter == null) continue;
            // filter is a base filter
            if(is_array($baseFilters) && $filter->baseFilter == 0 && $reportingSession->isUser()) {
                $baseFilters[$filterId] = "";
                continue;
            }
            if($filter->baseFilter > 0) {
                $baseFilters[$filter->baseFilter] = "";
            }
            if ($filter !== null) {
                $filters[$filter->id] = $filter;
                if ($groupBy != '') {
                    $cols = array();
                    $colNum = 0;
                    $cols[$colNum++] = new FilterColumn($groupBy, $colNum); // first column is always the format by col
                    $groupBySort = -1;
                    foreach ($filters[$filter->id]->columns as $col) {   // columns are already sorted by sort order, so this works
                        if ($col->name != $groupBy) {
                            if ($groupBySort == -1) {
                                $col->sortOrder++;
                            }
                            $cols[$colNum++] = $col;
                        } else {
                            $groupBySort = $col->sortOrder;
                        }
                    }
                    $filters[$filter->id]->columns = $cols;
                }
            } else {
                break;
            }
        }
        return $filters;
    }
    
    static function filterQueries($filters, &$qcond, &$isEventsReq, &$allTime, &$timeSet, &$filtName, &$logoPDF, &$footerPDF, &$filterReport) {
        foreach ($filters as $filter) {
            $filterReport .= $filter->name . ", ";
            if ($logoPDF === "" && $filter->logo !== "") {
                $logoPDF = $filter->logo;
            }
            if ($footerPDF === "" && $filter->footer !== "") {
                $footerPDF = $filter->footer;
            }
            if ($qcond != "") {
                $qcond .= " OR ";
            }
            if ($filtName == "") {
                $filtName = $filter->name;
            }
            self::repOrderCondFilter($filter, $qcond, $isEventsReq);
            if (count($filter->times) > 0) {
                $timeSet = true;
            } else {
                $allTime = true;
            }
        }
    }
    
    static function filterTimes($filters, &$query) {
        $query .= "AND (";
        foreach ($filters as $filter) {
            foreach ($filter->times as $time) {
                $dayCond = "";
                switch($time->day) {
                    case FilterTime::dayUnspecified: break;
                    case FilterTime::daySaSo: 
                        $dayCond = "(DATE_PART('DOW', local_stamp)=".(FilterTime::daySunday - FilterTime::daySunday)." OR DATE_PART('DOW', local_stamp)=".(FilterTime::daySaturday - FilterTime::daySunday).") AND ";
                        break;
                    case FilterTime::dayMonFri:
                        $dayCond = "(DATE_PART('DOW', local_stamp)>=".(FilterTime::dayMonday - FilterTime::daySunday)." AND DATE_PART('DOW', local_stamp)<=".(FilterTime::dayFriday - FilterTime::daySunday).") AND ";
                        break;
                    default:
                        $dayCond = "DATE_PART('DOW', local_stamp)=".($time->day - FilterTime::daySunday)." AND ";
                        break;
                }
                $query .= "($dayCond CAST(local_stamp AS TIME) >= time '" . $time->getFrom() . ":00' AND CAST(local_stamp AS TIME) <= time '" . $time->getTo() . ":59') OR ";
            }
        }
        $query = rtrim($query, " OR ");
        $query .= ") ";
    }
    
    static function create($remote, $dateStart, $dateEnd, $sortOrder, $sortBy, $groupBy, $oformat, $print, $pbx, $pbxObject, $tmpFilters, &$ajaxHref, &$pathToFile, &$error, $hideDigits, $billing, $lang, $mail = "", $filterInfo = false, $displayedDuration = ReportingConfigConsts::colCallDuration, $anonym = false) {
        /* @var $reportingSession ReportingSession */ global $reportingSession;
        if (($dateEnd + 86400) > 2147483648 || $dateStart > 2147483648) {
            $error = tl::tl("Make sure you select a year below 2038 (32-bit problem)");
            return false;
        }
        if ($dateStart > $dateEnd) {
            $error = tl::tlx('From date {$arg[1]} must be lower or equal to to date  {$arg[2]}.', InputPageDateField::translateDate($dateStart), InputPageDateField::translateDate($dateEnd));
            return false;
        }
        $sortOrder = (($sortOrder == "ascending" || !isset($sortOrder)) ? "ASC" : "DESC");
        $sortByVal = ReportingConfigConsts::getColName($sortBy);
        $baseFilters = array();
        $tmp = null;
        $error = "";
        $filters = self::parseFilters($tmpFilters, $baseFilters, $groupBy);
        $baseFilters = self::parseFilters($baseFilters, $tmp, $groupBy);
        foreach($baseFilters as /*@var $baseFilter Filter*/ $baseFilter) {
            if($baseFilter->anonym) {
                $anonym = true;
            }
        }
        if($anonym && ($hideDigits == 0 || $hideDigits == "")) {
            $hideDigits = 3;
        }
        if ($error != "") {
            return false;
        }
        if($reportingSession->isUser() && count($baseFilters) == 0) {
            $error = tl::tl("Select a base filter!");
            return false;
        }
        else if (!count($filters) && $pbxObject != '') {
            $filters[0] = new Filter();
            if (strcmp($pbxObject, "%"))
                $filters[0]->conds[0] = new FilterCondition(ReportingConfigConsts::colCCnUser, $pbxObject, FilterCondition::typeLocal);
            $cols = ReportingConfigConsts::getColNames(false);
            $colNum = 0;
            if ($groupBy != "") {
                $filters[0]->columns[$colNum++] = new FilterColumn($groupBy, $colNum);
            }
            if ($groupBy != ReportingConfigConsts::colLocalTime) {
                $filters[0]->columns[$colNum++] = new FilterColumn(ReportingConfigConsts::colLocalTime, $colNum);
            }
            $filters[0]->columns[$colNum++] = new FilterColumn(ReportingConfigConsts::colUnixTime, $colNum);
            foreach ($cols as $name => $col) {
                if ($name != ReportingConfigConsts::colLocalTime && $name != $groupBy) {    // time is already at first/second position
                    $filters[0]->columns[$colNum++] = new FilterColumn($name, $colNum);
                }
            }
            $filters[0]->name = "default";
        } else if (!count($filters)) {
            $error = tl::tl("Select a filter or define a PBX object!");
            return false;
        }
        
        $reportingDb = get_reporting_db();
        if($reportingSession->isUser()) {   // check if user is allowed to use the given filters
            $ids = "0";
            $idCount = 0;
            foreach($filters as $filterId => $filterName) {
                if($filterId > 0) {
                    $ids .= ",$filterId";
                    $idCount++;
                }
            }
            foreach($baseFilters as $filterId => $filterName) {
                $ids .= ",$filterId";
                $idCount++;
            }
            $query = "SELECT COUNT(uf.filter_id) AS c
                        FROM user_filters uf
                        WHERE uf.user_id={$reportingSession->user->id} AND uf.filter_id IN ($ids)";
            $result = $reportingDb->run($query);
            $row = pg_fetch_object($result);
            if(intval($row->c) != $idCount) {
                $error = tl::tl("You are not allowed to use all selected filters!");
                return false;
            }
        }

        $href = "?" . ReportingInputPage::valFrom . "=" . $dateStart .
                "&" . ReportingInputPage::valTo . "=" . $dateEnd .
                "&" . ReportingInputPage::valPbx . "=" . urlencode($pbx) .
                "&" . ReportingInputPage::valObject . "=" . urlencode($pbxObject) .
                "&" . ReportingInputPage::valBtnName . "=OK" .
                "&" . ReportingInputPage::valGroupBy . "[]=" . $groupBy .
                "&" . ReportingInputPage::valHideDigits . "=" . $hideDigits .
                "&lang[]=$lang" .
                "&" . ReportingInputPage::valDisplayedDuration . "[]=" . $displayedDuration .
                "&mode=" .
                "&_phase=validate" .
                "&_" . ReportingInputPage::valList . "_count=" . count($tmpFilters);
        if ($filterInfo) {
            $href .= "&" . ReportingInputPage::valFilterInfo . "[]=on";
        }
        if ($anonym) {
            $href .= "&" . ReportingInputPage::valAnonym . "[]=on";
        }
        $row = 0;
        foreach ($tmpFilters as $id => $filter) {
            $href .= "&" . ReportingInputPage::valList . "select@" . $row . "[]=" . $id;
            $row++;
        }

        $tmpDir = tempdir("/var/www/innovaphone/apps/innovaphone-reporting/tmp", "createdreport");
        $tmpConfig = $tmpDir . "/config.cfg";
        $href = $_SERVER["SCRIPT_NAME"] . $href;

        $tmpQuery = $oquery = $qcond = "";
        $isEventsReq = false;
        $reportColumns = array();
        $qcond = "";
        $filtName = "";
        $timeSet = $timeSetBase = false;
        $allTime = $allTimeBase = false;
        $logoPDF = "";
        $footerPDF = "";
        $filterReport = "";
        self::repOrderCols($tmpQuery, $oquery, $reportColumns, $groupBy, $sortByVal, $sortOrder, $displayedDuration);
        $qcondFilter = "";
        $qcondFilterBase = "";
        self::filterQueries($filters, $qcondFilter, $isEventsReq, $allTime, $timeSet, $filtName, $logoPDF, $footerPDF, $filterReport);
        if(count($baseFilters) > 0) {
            self::filterQueries($baseFilters, $qcondFilterBase, $isEventsReq, $allTimeBase, $timeSetBase, $filtName, $logoPDF, $footerPDF, $filterReport);
        }
        if($qcondFilter != "" && $qcondFilterBase != "") {
            $qcond = "(";
        }
        $qcond .= $qcondFilter;
        if($qcondFilterBase != "") {
            if($qcondFilter != "") {
                $qcond .= " AND ";
            }
            $qcond .= $qcondFilterBase;
        }
        if($qcondFilter != "" && $qcondFilterBase != "") {
            $qcond .= ") ";
        }
        $filterReport = substr($filterReport, 0, -2);
        if ($logoPDF !== "")
            $logoPDF = Config::imagesDir . "/" . $logoPDF;
        if ($footerPDF !== "")
            $footerPDF = Config::imagesDir . "/" . $footerPDF;
        // one filter without time condition overrides all other time conditions!
        if ($allTime)
            $timeSet = false;
        if ($allTimeBase) 
            $timeSetBase = false;
        if ($qcond != "")
            $qcond = " (" . $qcond . ") ";

        $query = "SELECT DISTINCT ";
        $query .= $tmpQuery;
        if ($isEventsReq)
            $query .= "JOIN events ev ON cdr.id = ev.cdr_id ";
        $query .= "JOIN cdr_users cu ON cdr.cdr_user_id=cu.id ";
        /* string comparisions is much faster than performing two atoi conversions plus integer comparision */
        $query .= "WHERE local_stamp >= '" . date('Y-m-d', $dateStart) . "' AND local_stamp < '" . date('Y-m-d', ($dateEnd + 86400)) . "' ";
        if ($timeSet) {
            self::filterTimes($filters, $query);
        }
        if($timeSetBase) {
            self::filterTimes($baseFilters, $query);
        }
        if ($billing === false)
            $query .= "AND transferred=0 AND forwarded=0 ";
        $query .= "AND cause_code != " . ReportingConfigConsts::codeNonSelUserClr . " ";  // this must be changed in the future due to alerting calls...
        if ($qcond != "")
            $query .= " AND " . $qcond;
        if(trim($pbxObject, "%") != "") {
            $query .= " AND ".ReportingConfigConsts::colCCnUser." ILIKE '".  pg_escape_string($pbxObject)."' ";
        }
        $query .= " AND cu.".$reportingSession->reportingCfg->pbx->identifier."='".pg_escape_string($pbx)."' ";
        $query .= " AND (licensed=true OR licensed IS NULL)";
        $query .= "ORDER BY " . $oquery;
        if($oformat == ReportingConfigConsts::outFormatXsl) {
            $query .= " LIMIT ".ReportingConfigConsts::reportXslLimit;
        }
        $tmpFile = "$tmpDir/report-$filtName.";
        switch ($oformat) {
            case ReportingConfigConsts::outFormatPdf: $tmpFile .= "pdf";
                break;
            case ReportingConfigConsts::outFormatCsv: $tmpFile .= "csv";
                break;
            case ReportingConfigConsts::outFormatXml:
            //case "xsl": $tmpFile .= "xml"; break;
            case ReportingConfigConsts::outFormatXsl:
                if ($pathToFile == "internal")
                    $tmpFile .= "xml";
                else
                    $tmpFile .= "gz";
                break;
        }
        $pathToFile = $tmpFile;
        $fp = fopen($tmpConfig, "w+");
        fwrite($fp, "[ CONN-DB ]\n");
        fwrite($fp, $reportingDb->getConnString() . "\n");
        fwrite($fp, "[ QUERY ]\n");
        fwrite($fp, $query . "\n");
        fwrite($fp, "[ HREF ]\n");
        fwrite($fp, $href . "&xmlfile=" . urlencode($tmpFile) . "\n");
        fwrite($fp, "[ FORMAT ]\n");
        fwrite($fp, $oformat . "\n");
        fwrite($fp, "[ SORT-ORDER ]\n");
        fwrite($fp, $sortOrder . "\n");
        fwrite($fp, "[ SORT-VALUE ]\n");
        fwrite($fp, $sortByVal . "\n");
        fwrite($fp, "[ GROUP-VALUE ]\n");
        if ($groupBy == ReportingConfigConsts::colLocalTime)
            fwrite($fp, "time\n");
        elseif ($groupBy == ReportingConfigConsts::colCCnUser)
            fwrite($fp, "obj\n");
        else
            fwrite($fp, $groupBy . "\n");
        if (($logoPDF !== "") && ($logoPDF !== "none")) {
            fwrite($fp, "[ LOGO ]\n");
            fwrite($fp, $logoPDF . "\n");
            list($width, $height, $type, $attr) = getimagesize($logoPDF);
            fwrite($fp, "[ LOGO-WIDTH ]\n");
            fwrite($fp, $width . "\n");
        }
        if (($footerPDF !== "") && ($footerPDF !== "none")) {
            fwrite($fp, "[ FOOTER ]\n");
            fwrite($fp, $footerPDF . "\n");
            list($width, $height, $type, $attr) = getimagesize($footerPDF);
            fwrite($fp, "[ FOOTER-WIDTH ]\n");
            fwrite($fp, $width . "\n");
        }
        fwrite($fp, "[ PRINT ]\n");
        fwrite($fp, $print . "\n");
        fwrite($fp, "[ SERVER ]\n");
        $protParts = explode("/", $_SERVER["SERVER_PROTOCOL"]);
        if (isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"] == "on") {
            $protParts[0] .= "s";
        }
        $server = strtolower($protParts[0]) . "://" . $_SERVER["HTTP_HOST"];
        fwrite($fp, $server . "\n");
        fwrite($fp, "[ REPORT-FILE ]\n");
        fwrite($fp, "$tmpFile\n");
        fwrite($fp, "[ REPORT-LIMIT ]\n");
        fwrite($fp, ReportingConfigConsts::reportXslLimit."\n");
        fwrite($fp, "[ COL-NAMES ]\n");
        fwrite($fp, count($reportColumns) . "\n");
        for ($mInd = 0; $mInd < count($reportColumns); $mInd++) {
            fwrite($fp, $reportColumns[$mInd] . "\n");
        }
        if ($hideDigits != 0) {
            fwrite($fp, "[ HIDE-DIGITS ]\n");
            fwrite($fp, $hideDigits . "\n");
        }
        fwrite($fp, "[ FILTER-NAME ]\n");
        fwrite($fp, $mail . "\n");
        fwrite($fp, "[ FILTERS ]\n");
        fwrite($fp, $filterReport . "\n");
        if ($pbxObject !== "") {
            fwrite($fp, "[ PBX-OBJECT ]\n");
            fwrite($fp, $pbxObject . "\n");
        }
        fwrite($fp, "[ PBX ]\n");
        fwrite($fp, $pbx. "\n");
        fwrite($fp, "[ FROM ]\n");
        fwrite($fp, InputPageDateField::translateDate($dateStart) . "\n");
        fwrite($fp, "[ TO ]\n");
        fwrite($fp, InputPageDateField::translateDate($dateEnd) . "\n");
        if ($filterInfo) {
            fwrite($fp, "[ FILTER-INFO ]\n");
            fwrite($fp, "1\n");
        }
        if ($displayedDuration) {
            fwrite($fp, "[ DISPLAYED-DURATION ]\n");
            fwrite($fp, "$displayedDuration\n");
        }
        if ($anonym) {
            fwrite($fp, "[ ANONYM ]\n");
            fwrite($fp, "1\n");
        }
        fwrite($fp, "[ LANGUAGE ]\n");
        fwrite($fp, "$lang\n");
        fclose($fp);
        $ajaxHref = str_replace("report.php", "report.fcgi", $_SERVER["SCRIPT_NAME"]) . "?cfg=" . urlencode(str_replace(Config::webRoot . "/" . Config::applicationsDir . "/innovaphone-reporting/tmp/", "", $tmpConfig));
        return true;
    }

    function action(InputPage &$page, $phase) {
        global $reportingSession;
        global $translator;
        $dateStart = $page->getField(ReportingInputPage::valFrom);
        $dateEnd = $page->getField(ReportingInputPage::valTo);
        $ajax = $page->getField("ajaxhref");
        $ajax->value = "";

        $sortByField = $page->getField(ReportingInputPage::valSortBy);
        $sortOrderField = $page->getField(ReportingInputPage::valSortOrder);
        $format = $page->getField(ReportingInputPage::valFormat);
        $chkFilterInfo = $page->getField(ReportingInputPage::valFilterInfo);
        $dispDuration = $page->getField(ReportingInputPage::valDisplayedDuration);
        $chkAnonym = $page->getField(ReportingInputPage::valAnonym);
        $oformat = array_search(true, $format->value);
        $sortOrder = null;
        $sortBy = null;
        if ($oformat == "") {
            $oformat = ReportingConfigConsts::outFormatXsl;
        }
        else {
            $sortOrder = ($sortOrderField->value["on"] ? "ascending" : "descending");
            $sortBy = array_search(true, $sortByField->value);
        }
        
        $print = InputPage::formInput("print");
        if ($print == "1") {
            $print = true;
        } else {
            $print = false;
        }
        $billing = InputPage::formInput(ReportingInputPage::valBillingCalls);
        if (!isset($billing))
            $billing = false;
        elseif ($billing == "true")
            $billing = true;
        else
            $billing = false;

        $numDigits = $page->getField(ReportingInputPage::valHideDigits);
        if (is_numeric($numDigits->value) && ($numDigits->value > 0))
            $hideDigits = $numDigits->value;
        else
            $hideDigits = 0;

        $xmlFile = InputPage::formInput("xmlfile");
        // reuse already existing xml file?
        if (isset($xmlFile) && $xmlFile != "" && file_exists($xmlFile) && $oformat != "pdf" && $oformat != "csv") {
            if ($this->readfile($xmlFile, $oformat, $print, $sortOrder, $sortBy) !== false) {
                return;
            }
        }

        $pbx = $page->getField(ReportingInputPage::valPbx);
        $pbxObject = $page->getField(ReportingInputPage::valObject);
        $groupBy = $page->getField(ReportingInputPage::valGroupBy);
        $groupByVal = array_search('true', $groupBy->value);
        $list = $page->getField(ReportingInputPage::valList);
        $filters = array();
        foreach ($list->value as $val) {
            $value = $val[$list->name . "select"];
            $id = intval(array_search(true, $value));
            if ($id != 0) {
                $filters[$id] = $id;
            }
        }
        $error = "";
        $ajaxHref = "";
        $pathToFile = "";
        $filterInfo = ($chkFilterInfo->value["on"] ? true : false);
        $anonym = ($chkAnonym->value["on"] ? true : false);
        if (self::create(false, $dateStart->value, $dateEnd->value, $sortOrder, $sortBy, $groupByVal, $oformat, $print, array_search(true, $pbx->value), $pbxObject->value, $filters, $ajaxHref, $pathToFile, $error, $hideDigits, $billing, $translator->currentLanguage(), "", $filterInfo, array_search(true, $dispDuration->value), $anonym) === false) {
            return $this->error($page, $error);
        }
        if ($oformat == ReportingConfigConsts::outFormatXsl) {
            $ajax->value = $ajaxHref;
            $page->links[count($page->links)] = array("rel" => "stylesheet", "type" => "text/css", "href" => "report.css");
            $page->render();
        } else {
            header("Location: $ajaxHref");
        }
    }

    private function readfile($file, $oformat, $print, $sortOrder = "", $sortBy = "") {
        $fileName = ltrim(strrchr($file, "/"), "/");
        $fileSize = filesize($file);
        header('Pragma: private');
        header('Cache-control: private, must-revalidate');
        if (!$print) {
            $gzFile = strpos($fileName, ".gz");
            if ($gzFile) {
                $fileName = substr($fileName, 0, $gzFile) . ".xml";
            }
            header("Content-Disposition: attachment; filename={$fileName}");
        }
        switch ($oformat) {
            case ReportingConfigConsts::outFormatPdf: header('Content-type: application/pdf');
                break;
            case ReportingConfigConsts::outFormatCsv: header('Content-type: text/csv');
                break;
            case ReportingConfigConsts::outFormatXml:
            case ReportingConfigConsts::outFormatXsl: header('Content-type: text/xml');
                break;
        }
        if (!$print) {
            if ($oformat == ReportingConfigConsts::outFormatPdf || $oformat == ReportingConfigConsts::outFormatCsv) {
                header('Content-len: ' . $fileSize);
                return @readfile($file);
            } else if (substr_count($_SERVER['HTTP_ACCEPT_ENCODING'], 'deflate')) {
                header('Content-Encoding: deflate');
                header('Content-len: ' . $fileSize - 2);
                $fh = fopen($file, "r");
                fseek($fh, 2);
                if (!$fh)
                    return false;
                while (!feof($fh)) {
                    $read = fread($fh, 1024);
                    echo $read;
                }
            } else {
                $fh = fopen($file, "r");
                if (!$fh)
                    return false;
                while (!feof($fh)) {
                    $read = fread($fh, 1024);
                    $uncompressed = gzuncompress($read);
                    echo $uncompressed;
                }
            }
        } else {
            $tmpXml = tempnam("/tmp", "report-xml");
            if (!$tmpXml)
                return false;
            $cmd = "/usr/local/bin/zpipe -d < " . $file . " > " . $tmpXml . " 2>&1";
            $ret = exec($cmd, $retval);
            if ($ret != 0) {
                unlink($tmpXml);
                return false;
            }
            $fh = fopen($tmpXml, "r");
            if (!$fh)
                return false;
            if($sortOrder == "ASC") $sortOrder = "ascending";
            elseif($sortOrder == "DESC") $sortOrder = "descending";
            $uncompressed = fread($fh, 8192);
            $search = "<report ";
            if (($reportPos = strpos($uncompressed, $search)) !== false) {
                if (strpos($uncompressed, "{$search}print='true'") === false && strpos($uncompressed, "{$search}print=\"true\"") === false) {
                    $uncompressed = str_insert("print='true' ", $uncompressed, $reportPos + strlen($search));
                }
                $uncompressed = preg_replace("!sort-by='[^']*'!is", "sort-by='$sortBy'", $uncompressed, 1, $count);
                if ($count == 0) {
                    $uncompressed = preg_replace("!sort-by=\"[^\"]*\"!is", "sort-by=\"$sortBy\"", $uncompressed, 1, $count);
                }
                $uncompressed = preg_replace("!sort-order='[^']*'!is", "sort-order='$sortOrder'", $uncompressed, 1, $count);
                if ($count == 0) {
                    $uncompressed = preg_replace("!sort-order=\"[^\"]*\"!is", "sort-order=\"$sortOrder\"", $uncompressed, 1, $count);
                }
                echo $uncompressed;
                while (!feof($fh)) {
                    echo fread($fh, 8192);
                }
                fclose($fh);
                unlink($tmpXml);
                return true;
            } else {
                fclose($fh);
                unlink($tmpXml);
                return false;
            }
        }
    }

    private function error(InputPage &$page, $error) {
        $this->msg = $error;
        $page->render();
        return false;
    }

}

class PageReport extends MyPage {

    public function getHelpAnchor() {
        switch ($this->parent->currentTab) {
            case ReportingReportConsts::tabCalls:
                return "#Web_Interface";
            default:
                return "#Report_Creation";
        }
    }

    public function getDefaultTab() {
        return ReportingReportConsts::tabCalls;
    }

    public function getHead() {
        return "";
    }

    public function createFields() {
        $this->addSection(ReportingReportConsts::tabCalls, 3);

        $genericHint = "";

        $this->parent->addTabMenuItem(tl::tl("Calls"), "javascript:submitTab('" . ReportingReportConsts::tabCalls . "')", $this->parent->currentTab == ReportingReportConsts::tabCalls);

        switch ($this->parent->currentTab) {
            case ReportingReportConsts::tabCalls:
                $this->tabCalls();
                break;
        }
    }

    public function tabCalls() {
        $this->currentSection = $this->parent->currentTab;

        $error = InputPage::formInput("error");
        if (isset($error)) {
            $errorField = $this->addField(new InputPageText("error", ""));
            $errorField->value = "<b>" . tl::tl("An unexpected error occurred:") . "</b>";
            $errorField->value .= "<br><div style=\"color:red\">$error</div><br><br>";
            $errorField->fullline = true;
        }

        global $reportingSession;

        $this->parent->addObjectAndFilterList($this);

        $this->addSubSection($this->currentSection, "dates", 2);

        $dateStart = $this->addField(new InputPageCalendarDateField(ReportingInputPage::valFrom, tl::tl("From")));
        $dateEnd = $this->addField(new InputPageCalendarDateField(ReportingInputPage::valTo, tl::tl("To")));

        $this->parent->addFieldGroupBy($this);
        $dispDuration = $this->parent->addFieldDisplayedDuration($this);
        $dispDuration->fullline = true;
        $hide = $this->parent->addFieldHideDigit($this);
        $anonym = $this->parent->addFieldAnonym($this);
        $hide->fullline = $anonym->fullline = false;
        
        $chkFilterInfo = $this->addField(new InputPageCheckBoxField(ReportingInputPage::valFilterInfo, tl::tl("Filter Info")));
        $chkFilterInfo->fullline = true;
        $default = "";
        $options = array("" => "",
                            ReportingConfigConsts::outFormatPdf => "PDF", 
                            ReportingConfigConsts::outFormatXml => "XML", 
                            ReportingConfigConsts::outFormatCsv => "CSV");
        $this->addField($directDownload = new InputPageDropdownField(ReportingInputPage::valFormat, tl::tl("Direct Download As"), $options));
        $format = InputPage::formInput(ReportingInputPage::valFormat);
        if(isset($format) && !is_array($format)) {
            $default = $format;
        }
        $directDownload->default[$format] = true;
        $directDownload->hint = tl::tl("Leave empty if you wish to view the report in your browser or select your desired download format.");
        $directDownload->fullline = true;
        $sortBy = $this->parent->addFieldSortBy($this);
        $sortByVal = InputPage::formInput(ReportingInputPage::valSortBy);
        if(isset($sortByVal) && !is_array($sortByVal)) {
            $sortBy->default = array();
            $sortBy->default[$sortByVal] = true;
        }
        $sortOrder = $this->parent->addFieldCheckBoxSortOrder($this, "ASC");
        $sortOrderVal = InputPage::formInput(ReportingInputPage::valSortOrder);
        if(isset($sortOrderVal) && !is_array($sortOrderVal)) {
            $sortOrder->default = array();
            $sortOrder->default["on"] = (($sortOrderVal == "ASC" || $sortOrderVal == "ascending") ? true : false);
        }
        $java = "changeDirectDownload('{$directDownload->name}', '{$sortBy->name}', '{$sortOrder->name}')";
        $directDownload->attributes["onChange"] = "javascript:$java";
        $directDownload->setJavaScriptInit("function JavaScriptInit_{$directDownload->name}() { $java }");
        
        $btn = $this->addField(new ButtonCreateReport(ReportingInputPage::valBtnName, tl::tl("Ok")));
        $btn->fullline = true;
        $btn->tdattributes["style"] = "padding-top:10px";

        $this->addSection("display", 1, false);
        $this->currentSection = "display";
        $displayXml = $this->addField(new InputPageText("displayxml", ""));
        $displayXml->attributes["id"] = $displayXml->name;

        $ajaxHref = $this->addField(new InputPageHiddenIdField("ajaxhref", ""));
        $btn->setJavaScriptInit("function JavaScriptInit_{$btn->name}() { parseXML('{$ajaxHref->name}', '{$displayXml->name}', '') }");
    }

    public function fillFilterList() {
        $list = $this->parent->getField(ReportingInputPage::valList);
        $rows = 0;
        global $reportingSession;
        $userId = 0;
        if($reportingSession->isUser()) {
            $userId = $reportingSession->user->id;
        }
        $selects = $this->parent->getFilterListArray(false, $userId);

        if (!count($list->value)) {
            $list->value = array();
            $list->value[0][$list->name . "select"][0] = true;
            $list->value[0][$list->name . "select_options"] = $selects;
        } else {
            foreach ($list->value as $key => &$vals) {
                $vals[$list->name . "select_options"] = $selects;
                $rows++;
            }
        }
    }

    public function fillLists() {
        switch ($this->parent->currentTab) {
            case ReportingReportConsts::tabCalls:
                switch ($this->parent->currentMode) {
                    case "":
                        $this->fillFilterList();
                        break;
                }
                break;
        }
    }

}

// includes the login fields and links for help/register
final class ReportingReport extends ReportingInputPage {

    // constructor
    public function ReportingReport() {
        parent::ReportingInputPage(tl::tl("Reporting Reports"), "reportingreport");
        $this->setFieldSectionAttributes("inline", new _InputPageFieldSectionAttributes(array(), 1));

        global $reportingSession;

        // add proper fields to pages
        $this->pages = array(ReportingReportConsts::pageReport => new PageReport($this, false));

        // update page
        $this->setDefaultTabProperties();
        $this->pages[$this->pageNo]->createFields();
    }

    public function getArgs() {
        return "tab={$this->currentTab}";
    }

}

;

/*
  report.php can be used as "remote" interface with human readable parameters
  so some parameters will be set differently
 */
$remote = InputPage::formInput("remote");
if (isset($remote)) {    // enable remote interface
    if(ReportingMethods::isUserPath()) {
        set_header_error(tl::tl("No remote interface for non admin users available!"));
        return;
    }
    global $reportingSession;
    $reportingDb = get_reporting_db();
    /* unchanged parameters */
    $mail = InputPage::formInput(ReportingInputPage::valMailName);
    $sortBy = InputPage::formInput(ReportingInputPage::valSortBy);
    $sortOrder = InputPage::formInput(ReportingInputPage::valSortOrder);
    $from = InputPage::formInput(ReportingInputPage::valFrom);
    $to = InputPage::formInput(ReportingInputPage::valTo);
    $format = InputPage::formInput(ReportingInputPage::valFormat);
    $pbxObject = InputPage::formInput(ReportingInputPage::valObject);
    $pbx = InputPage::formInput(ReportingInputPage::valPbx);
    $groupBy = InputPage::formInput(ReportingInputPage::valGroupBy);
    $hideDigits = InputPage::formInput(ReportingInputPage::valHideDigits);
    $billing = InputPage::formInput(ReportingInputPage::valBillingCalls);
    $filterInfo = InputPage::formInput(ReportingInputPage::valFilterInfo);
    $displayedDuration = InputPage::formInput(ReportingInputPage::valDisplayedDuration);
    $anonym = InputPage::formInput(ReportingInputPage::valAnonym);
    $bulk = InputPage::formInput(ReportingInputPage::valBulk);
    $lang = InputPage::formInput("lang");
    if (!isset($lang)) {
        $lang = "en";
    }
    $pathToFile = "";
    /* @var $repMail ReportMail*/ $repMail = null;
    if (isset($mail)) {
        $repMail = ReportingMethods::getReportMail(0, $mail, $error);
        if ($repMail != null) {
            $hideDigits = $repMail->hideDigits;
            $format = $repMail->format;
            $mailsAddr = explode(",", $repMail->mails);
            $groupBy = $repMail->groupBy;
            $sortBy = $repMail->sortBy;
            $sortOrder = $repMail->sortOrder;
            $filterName = $repMail->name;
            $pbx = $repMail->pbx;
            $pbxObject = $repMail->pbxObject;
            $lang = $repMail->lang;
            $filters = $repMail->filters;
            $displayedDuration = $repMail->displayedDuration;
            $anonym = $repMail->anonym;
            $bulk = $repMail->bulk;
            $testMail = InputPage::formInput("testmail");
            $toString = "yesterday";
            if(isset($testMail)) $toString = "today";
            $to = strtotime(date('Y-m-d', strtotime($toString)));
            switch ($repMail->interval) {
                case 0: // daily
                    $from = $to;
                    if(isset($testMail)) {
                        $from = strtotime(date('Y-m-d', strtotime("yesterday")));
                    }
                    break;
                case 1: // weekly
                    $from = strtotime(date('Y-m-d', strtotime("-1week")));
                    break;
                case 2: // monthly
                    $from = strtotime(date('Y-m-d', strtotime("-1month")));
                    break;
            }
        }
        $pathToFile = "internal";
    } else {
        if($displayedDuration == "") {
            $displayedDuration = ReportingConfigConsts::colCallDuration;
        }
        if ($billing == "true")
            $billing = true;
        else
            $billing = false;

        if ($filterInfo == "true")
            $filterInfo = true;
        else
            $filterInfo = false;

        if ($anonym == "true")
            $anonym = true;
        else
            $anonym = false;
        
        if($bulk == "true") {
            $bulk = true;
        }
        else {
            $bulk = false;
        }
        
        if (!isset($hideDigits))
            $hideDigits = 0;
        else 
            $hideDigits = intval($hideDigits);
        if (!isset($format))
            $format = "xml";
        if (!isset($groupBy))
            $groupBy = "";
        elseif($groupBy == "cn") {  // compatibility to previous definition in V1
            $groupBy = ReportingConfigConsts::colCCnUser;
        }
        if (!isset($from) || !isset($to)) {
            set_header_error(tl::tlx('You have to set the parameters \'{$arg[1]}\' and \'{$arg[2]}\'.', ReportingInputPage::valFrom, ReportingInputPage::valTo));
            return;
        }
        if (!preg_match("/^[0-9]+$/", $from)) {
            $from = strtotime($from);
        }
        if (!preg_match("/^[0-9]+$/", $to)) {
            $to = strtotime($to);
        }
        if ($to === false || $from === false) {
            set_header_error(tl::tlx('You have to use a timestamp or the format \'{$arg[1]}\' for the parameters \'{$arg[2]}\' and \'{$arg[3]}\'.', MainInputPage::dateFormat, ReportingInputPage::valFrom, ReportingInputPage::valTo));
            return;
        }
        /* now the filters */
        $filterQuery = "SELECT id, name FROM filters WHERE ";
        $filters = array();
        $i = 1;
        do {
            $name = InputPage::formInput("filter$i");
            if (isset($name)) {
                $filterQuery .= "name ILIKE '" . pg_escape_string($name) . "' OR ";
            }
            $i++;
        } while (isset($name));
        if (strpos($filterQuery, " OR ") !== false) {
            $filterQuery = rtrim($filterQuery, " OR ");
            $result = $reportingDb->run($filterQuery);
            while ($row = pg_fetch_object($result)) {
                $filters[$row->id] = $row->name;
            }
        }
    }
    if(!isset($pbx)) {
        set_header_error(tl::tlx('Set the parameter \'{$arg[1]}\'!', ReportingInputPage::valPbx));
        return;
    }
    if (!count($filters) && (!isset($pbxObject) || $pbxObject == "")) {
        set_header_error(tl::tlx('Set a filterX parameter or the parameter \'{$arg[1]}\'!', ReportingInputPage::valObject));
        return;
    }
    $ajaxHref = "";
    $error = "";
    tl::change($lang);
    $mails = array();
    if(isset($mail) && $bulk) {
        $result = ReportingPbxObjectField::getUsers($pbxObject, $pbx);
        while(($row = pg_fetch_object($result))) {
            if(valid_email($row->email)) {
                $mails[$row->id] = $row;
            }
        }
        if(!count($mails)) {
            set_header_error(tl::tl("No matching user found!"));
            return;
        }        
    }
    else {
        $mails[0] = "";
    }
    foreach($mails as $mailId => $row) {
        if($mailId > 0) {
            $pbxObject = $row->cn;
        }
        if (ButtonCreateReport::create(true, $from, $to, $sortOrder, $sortBy, $groupBy, $format, false, $pbx, $pbxObject, $filters, $ajaxHref, $pathToFile, $error, $hideDigits, $billing, $lang, $mail, $filterInfo, $displayedDuration, $anonym) === false) {
            set_header_error($error);
        } else {
            if (isset($mail)) {
                system("cd /var/www/innovaphone; ./process.fcgi $ajaxHref; cd -;");
                $reportTmp = "";
                if($repMail->compress) {
                    $tmpDir = tempdir("/tmp", "mailreport");
                    $reportTmp = "$tmpDir/report-$mail.gz";
                    $reportTmp = str_replace(" ", "", $reportTmp);
                    system("gzip -9c \"$pathToFile\" > \"$reportTmp\"");
                }
                else {
                    $reportTmp = $pathToFile;
                    $reportTmp = str_replace("default", $mail, $reportTmp);
                    $reportTmp = str_replace(" ", "", $reportTmp);
                    rename($pathToFile, $reportTmp);
                }
                $testMail = "";
                if($mailId > 0) {
                    $mailsAddr[] = $row->email;
                }
                foreach ($mailsAddr as $addr) {
                    if ($reportingSession->mainSession->webCfg->smtprelay->active == "true") {
                        $cmd = "/usr/bin/sudo /usr/local/bin/config.sh mails-smtp";
                        $cmd.=" -pr " . $reportingSession->mainSession->webCfg->smtprelay->prot;
                        if($reportingSession->mainSession->webCfg->smtprelay->user) {
                            $cmd.=" -u " . $reportingSession->mainSession->webCfg->smtprelay->user;
                            if ($reportingSession->mainSession->webCfg->smtprelay->password)
                                $cmd.=" -p " . $reportingSession->mainSession->webCfg->smtprelay->password;
                        }
                        $cmd.=" -s " . $reportingSession->mainSession->webCfg->smtprelay->server . " -pt " . $reportingSession->mainSession->webCfg->smtprelay->port;
                        $cmd.=" -m " . $reportingSession->mainSession->webCfg->smtprelay->mail;
                        $cmd.=" -sb \"" . $filterName . "\" -a " . $reportTmp . " -d " . $addr . " 2>&1";
                    }
                    else {
                        $cmd = "/usr/bin/sudo /usr/local/bin/config.sh mails -sb \"" . $filterName . "\" -a " . $reportTmp . " -d " . $addr;
                    }
                    $res = exec($cmd, $retval);
                }
                if($mailId > 0) {
                    unset($mailsAddr[array_search($row->email, $mailsAddr)]);
                }
                if($repMail->compress) {
                    unlink($reportTmp);
                    rmdir($tmpDir);
                }
            } else {
                header("Location: $ajaxHref");
            }
        }
    }
    close_reporting_db();
    return;
}

// create page
$page = new ReportingReport;

if ($page->pageNo == ReportingReportConsts::pageReport) {
    $page->updateFields();
    $error = InputPage::formInput("error");
    if (!isset($error)) {
        $page->pages[$page->pageNo]->fillLists();
    }
}

$page->work();

close_reporting_db();
?>
